import * as $ from '@manahippo/move-to-ts';
import { AptosDataCache, AptosParserRepo, AptosLocalCache } from '@manahippo/move-to-ts';
import { U8, U64, U128 } from '@manahippo/move-to-ts';
import { TypeParamDeclType, FieldDeclType } from '@manahippo/move-to-ts';
import { StructTag, TypeTag } from '@manahippo/move-to-ts';
import { OptionTransaction } from '@manahippo/move-to-ts';
import { HexString, AptosClient, AptosAccount, TxnBuilderTypes, Types } from 'aptos';
import * as Stdlib from '../stdlib';
export declare const packageName = "HippoAggregator";
export declare const moduleAddress: HexString;
export declare const moduleName = "volume";
export declare const E_NOT_ADMIN: U64;
export declare const E_NOT_POSTER: U64;
export declare const E_REPEAT_POST: U64;
export declare const E_VERCTOR_LENGT_NOT_EQUAL: U64;
export declare const VOLUME_HISTORY_LENGTH: U64;
export declare class PoolProvider {
    typeTag: TypeTag;
    static moduleAddress: HexString;
    static moduleName: string;
    __app: $.AppType | null;
    static structName: string;
    static typeParameters: TypeParamDeclType[];
    static fields: FieldDeclType[];
    dex_type: U8;
    amount: U64;
    constructor(proto: any, typeTag: TypeTag);
    static PoolProviderParser(data: any, typeTag: TypeTag, repo: AptosParserRepo): PoolProvider;
    static getTag(): StructTag;
    loadFullState(app: $.AppType): Promise<void>;
}
export declare class TotalVolume {
    typeTag: TypeTag;
    static moduleAddress: HexString;
    static moduleName: string;
    __app: $.AppType | null;
    static structName: string;
    static typeParameters: TypeParamDeclType[];
    static fields: FieldDeclType[];
    start_time: U64;
    amount: U64;
    constructor(proto: any, typeTag: TypeTag);
    static TotalVolumeParser(data: any, typeTag: TypeTag, repo: AptosParserRepo): TotalVolume;
    static getTag(): StructTag;
    loadFullState(app: $.AppType): Promise<void>;
}
export declare class TradingPair {
    typeTag: TypeTag;
    static moduleAddress: HexString;
    static moduleName: string;
    __app: $.AppType | null;
    static structName: string;
    static typeParameters: TypeParamDeclType[];
    static fields: FieldDeclType[];
    coin_x: Stdlib.String.String;
    coin_y: Stdlib.String.String;
    amount: U64;
    constructor(proto: any, typeTag: TypeTag);
    static TradingPairParser(data: any, typeTag: TypeTag, repo: AptosParserRepo): TradingPair;
    static getTag(): StructTag;
    loadFullState(app: $.AppType): Promise<void>;
}
export declare class Volume {
    typeTag: TypeTag;
    static moduleAddress: HexString;
    static moduleName: string;
    __app: $.AppType | null;
    static structName: string;
    static typeParameters: TypeParamDeclType[];
    static fields: FieldDeclType[];
    poster: HexString;
    total_volume: U128;
    last_24h_volume: U64;
    last_7d_volume: U64;
    data_end_sequence_number: U64;
    data_end_time: U64;
    volume_decimals: U64;
    total_volume_history_24h: TotalVolume[];
    total_volume_history_7d: TotalVolume[];
    top_trading_pairs_24h: TradingPair[];
    top_trading_pairs_7d: TradingPair[];
    top_pool_provider_24h: PoolProvider[];
    top_pool_provider_7d: PoolProvider[];
    constructor(proto: any, typeTag: TypeTag);
    static VolumeParser(data: any, typeTag: TypeTag, repo: AptosParserRepo): Volume;
    static load(repo: AptosParserRepo, client: AptosClient, address: HexString, typeParams: TypeTag[]): Promise<Volume>;
    static loadByApp(app: $.AppType, address: HexString, typeParams: TypeTag[]): Promise<Volume>;
    static getTag(): StructTag;
    loadFullState(app: $.AppType): Promise<void>;
}
export declare function add_volume_(total_volume_array: TotalVolume[], round_start_time: U64, amount: U64, $c: AptosDataCache): void;
export declare function clean_(poster: HexString, $c: AptosDataCache): void;
export declare function buildPayload_clean(isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
export declare function fetch_volume_(fetcher: HexString, $c: AptosDataCache): void;
export declare function buildPayload_fetch_volume(isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
export declare function query_fetch_volume(client: AptosClient, fetcher: $.SimulationKeys, repo: AptosParserRepo, $p: TypeTag[], option?: OptionTransaction, _isJSON?: boolean): Promise<Volume>;
export declare function get_volume_($c: AptosDataCache): Volume;
export declare function buildPayload_get_volume(isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
export declare function initialize_(admin: HexString, poster: HexString, $c: AptosDataCache): void;
export declare function buildPayload_initialize(poster: HexString, isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
export declare function parse_pool_provider_vector_(dex_type_vector: U8[], amount_vector: U64[], $c: AptosDataCache): PoolProvider[];
export declare function parse_trading_pairs_vector_(coin_x_vector: U8[][], coin_y_vector: U8[][], amount_vector: U64[], $c: AptosDataCache): TradingPair[];
export declare function parse_volume_history_vector_(start_time_vector: U64[], amount_vector: U64[], $c: AptosDataCache): TotalVolume[];
export declare function post_(_poster: HexString, _amount: U64, _last_24_volume: U64, _last_7d_volume: U64, _round_start_time_24h: U64, _round_start_time_7d: U64, _new_data_end_time: U64, _new_data_end_seauence_number: U64, _trading_pairs_24h_coin_x: U8[][], _trading_pairs_24h_coin_y: U8[][], _trading_pairs_24h_amount: U64[], _trading_pairs_7d_coin_x: U8[][], _trading_pairs_7d_coin_y: U8[][], _trading_pairs_7d_amount: U64[], _pool_provider_24h_dex_type: U8[], _pool_provider_24h_amount: U64[], _pool_provider_7d_dex_type: U8[], _pool_provider_7d_amount: U64[], $c: AptosDataCache): void;
export declare function buildPayload_post(_amount: U64, _last_24_volume: U64, _last_7d_volume: U64, _round_start_time_24h: U64, _round_start_time_7d: U64, _new_data_end_time: U64, _new_data_end_seauence_number: U64, _trading_pairs_24h_coin_x: U8[][], _trading_pairs_24h_coin_y: U8[][], _trading_pairs_24h_amount: U64[], _trading_pairs_7d_coin_x: U8[][], _trading_pairs_7d_coin_y: U8[][], _trading_pairs_7d_amount: U64[], _pool_provider_24h_dex_type: U8[], _pool_provider_24h_amount: U64[], _pool_provider_7d_dex_type: U8[], _pool_provider_7d_amount: U64[], isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
export declare function post_v2_(poster: HexString, total_volume: U128, last_24_volume: U64, last_7d_volume: U64, data_end_time: U64, data_end_seauence_number: U64, total_volume_history_24h_start_time: U64[], total_volume_history_24h_volume: U64[], total_volume_history_7d_start_time: U64[], total_volume_history_7d_volume: U64[], trading_pairs_24h_coin_x: U8[][], trading_pairs_24h_coin_y: U8[][], trading_pairs_24h_amount: U64[], trading_pairs_7d_coin_x: U8[][], trading_pairs_7d_coin_y: U8[][], trading_pairs_7d_amount: U64[], pool_provider_24h_dex_type: U8[], pool_provider_24h_amount: U64[], pool_provider_7d_dex_type: U8[], pool_provider_7d_amount: U64[], $c: AptosDataCache): void;
export declare function buildPayload_post_v2(total_volume: U128, last_24_volume: U64, last_7d_volume: U64, data_end_time: U64, data_end_seauence_number: U64, total_volume_history_24h_start_time: U64[], total_volume_history_24h_volume: U64[], total_volume_history_7d_start_time: U64[], total_volume_history_7d_volume: U64[], trading_pairs_24h_coin_x: U8[][], trading_pairs_24h_coin_y: U8[][], trading_pairs_24h_amount: U64[], trading_pairs_7d_coin_x: U8[][], trading_pairs_7d_coin_y: U8[][], trading_pairs_7d_amount: U64[], pool_provider_24h_dex_type: U8[], pool_provider_24h_amount: U64[], pool_provider_7d_dex_type: U8[], pool_provider_7d_amount: U64[], isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
export declare function set_poster_(admin: HexString, new_poster: HexString, $c: AptosDataCache): void;
export declare function buildPayload_set_poster(new_poster: HexString, isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
export declare function loadParsers(repo: AptosParserRepo): void;
export declare class App {
    client: AptosClient;
    repo: AptosParserRepo;
    cache: AptosLocalCache;
    constructor(client: AptosClient, repo: AptosParserRepo, cache: AptosLocalCache);
    get moduleAddress(): HexString;
    get moduleName(): string;
    get PoolProvider(): typeof PoolProvider;
    get TotalVolume(): typeof TotalVolume;
    get TradingPair(): typeof TradingPair;
    get Volume(): typeof Volume;
    loadVolume(owner: HexString, loadFull?: boolean, fillCache?: boolean): Promise<Volume>;
    payload_clean(isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
    clean(_account: AptosAccount, option?: OptionTransaction, _isJSON?: boolean): Promise<Types.UserTransaction>;
    payload_fetch_volume(isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
    fetch_volume(_account: AptosAccount, option?: OptionTransaction, _isJSON?: boolean): Promise<Types.UserTransaction>;
    query_fetch_volume($p: TypeTag[], option?: OptionTransaction, _isJSON?: boolean, fetcher?: $.SimulationKeys): Promise<Volume>;
    payload_get_volume(isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
    get_volume(_account: AptosAccount, option?: OptionTransaction, _isJSON?: boolean): Promise<Types.UserTransaction>;
    payload_initialize(poster: HexString, isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
    initialize(_account: AptosAccount, poster: HexString, option?: OptionTransaction, _isJSON?: boolean): Promise<Types.UserTransaction>;
    payload_post(_amount: U64, _last_24_volume: U64, _last_7d_volume: U64, _round_start_time_24h: U64, _round_start_time_7d: U64, _new_data_end_time: U64, _new_data_end_seauence_number: U64, _trading_pairs_24h_coin_x: U8[][], _trading_pairs_24h_coin_y: U8[][], _trading_pairs_24h_amount: U64[], _trading_pairs_7d_coin_x: U8[][], _trading_pairs_7d_coin_y: U8[][], _trading_pairs_7d_amount: U64[], _pool_provider_24h_dex_type: U8[], _pool_provider_24h_amount: U64[], _pool_provider_7d_dex_type: U8[], _pool_provider_7d_amount: U64[], isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
    post(_account: AptosAccount, _amount: U64, _last_24_volume: U64, _last_7d_volume: U64, _round_start_time_24h: U64, _round_start_time_7d: U64, _new_data_end_time: U64, _new_data_end_seauence_number: U64, _trading_pairs_24h_coin_x: U8[][], _trading_pairs_24h_coin_y: U8[][], _trading_pairs_24h_amount: U64[], _trading_pairs_7d_coin_x: U8[][], _trading_pairs_7d_coin_y: U8[][], _trading_pairs_7d_amount: U64[], _pool_provider_24h_dex_type: U8[], _pool_provider_24h_amount: U64[], _pool_provider_7d_dex_type: U8[], _pool_provider_7d_amount: U64[], option?: OptionTransaction, _isJSON?: boolean): Promise<Types.UserTransaction>;
    payload_post_v2(total_volume: U128, last_24_volume: U64, last_7d_volume: U64, data_end_time: U64, data_end_seauence_number: U64, total_volume_history_24h_start_time: U64[], total_volume_history_24h_volume: U64[], total_volume_history_7d_start_time: U64[], total_volume_history_7d_volume: U64[], trading_pairs_24h_coin_x: U8[][], trading_pairs_24h_coin_y: U8[][], trading_pairs_24h_amount: U64[], trading_pairs_7d_coin_x: U8[][], trading_pairs_7d_coin_y: U8[][], trading_pairs_7d_amount: U64[], pool_provider_24h_dex_type: U8[], pool_provider_24h_amount: U64[], pool_provider_7d_dex_type: U8[], pool_provider_7d_amount: U64[], isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
    post_v2(_account: AptosAccount, total_volume: U128, last_24_volume: U64, last_7d_volume: U64, data_end_time: U64, data_end_seauence_number: U64, total_volume_history_24h_start_time: U64[], total_volume_history_24h_volume: U64[], total_volume_history_7d_start_time: U64[], total_volume_history_7d_volume: U64[], trading_pairs_24h_coin_x: U8[][], trading_pairs_24h_coin_y: U8[][], trading_pairs_24h_amount: U64[], trading_pairs_7d_coin_x: U8[][], trading_pairs_7d_coin_y: U8[][], trading_pairs_7d_amount: U64[], pool_provider_24h_dex_type: U8[], pool_provider_24h_amount: U64[], pool_provider_7d_dex_type: U8[], pool_provider_7d_amount: U64[], option?: OptionTransaction, _isJSON?: boolean): Promise<Types.UserTransaction>;
    payload_set_poster(new_poster: HexString, isJSON?: boolean): TxnBuilderTypes.TransactionPayloadEntryFunction | Types.TransactionPayload_EntryFunctionPayload;
    set_poster(_account: AptosAccount, new_poster: HexString, option?: OptionTransaction, _isJSON?: boolean): Promise<Types.UserTransaction>;
}
//# sourceMappingURL=volume.d.ts.map