import * as $ from '@manahippo/move-to-ts';
import { AptosDataCache, AptosParserRepo, AptosLocalCache } from '@manahippo/move-to-ts';
import { U64, U128 } from '@manahippo/move-to-ts';
import { TypeParamDeclType, FieldDeclType } from '@manahippo/move-to-ts';
import { StructTag, TypeTag } from '@manahippo/move-to-ts';
import { HexString, AptosClient } from 'aptos';
import * as Stdlib from '../stdlib';
export declare const packageName = "Liquidswap";
export declare const moduleAddress: HexString;
export declare const moduleName = "liquidity_pool";
export declare const ERR_DISABLED: U64;
export declare const ERR_EMERGENCY: U64;
export declare const ERR_POOL_DOES_NOT_EXIST: U64;
export declare const ERR_POOL_IS_LOCKED: U64;
export declare const ERR_WRONG_PAIR_ORDERING: U64;
export declare const FEE_SCALE: U64;
export declare class IsEmergency {
    typeTag: TypeTag;
    static moduleAddress: HexString;
    static moduleName: string;
    __app: $.AppType | null;
    static structName: string;
    static typeParameters: TypeParamDeclType[];
    static fields: FieldDeclType[];
    constructor(proto: any, typeTag: TypeTag);
    static IsEmergencyParser(data: any, typeTag: TypeTag, repo: AptosParserRepo): IsEmergency;
    static load(repo: AptosParserRepo, client: AptosClient, address: HexString, typeParams: TypeTag[]): Promise<IsEmergency>;
    static loadByApp(app: $.AppType, address: HexString, typeParams: TypeTag[]): Promise<IsEmergency>;
    static getTag(): StructTag;
    loadFullState(app: $.AppType): Promise<void>;
}
export declare class LiquidityPool {
    typeTag: TypeTag;
    static moduleAddress: HexString;
    static moduleName: string;
    __app: $.AppType | null;
    static structName: string;
    static typeParameters: TypeParamDeclType[];
    static fields: FieldDeclType[];
    coin_x_reserve: Stdlib.Coin.Coin;
    coin_y_reserve: Stdlib.Coin.Coin;
    last_block_timestamp: U64;
    last_price_x_cumulative: U128;
    last_price_y_cumulative: U128;
    lp_mint_cap: Stdlib.Coin.MintCapability;
    lp_burn_cap: Stdlib.Coin.BurnCapability;
    x_scale: U64;
    y_scale: U64;
    locked: boolean;
    fee: U64;
    dao_fee: U64;
    constructor(proto: any, typeTag: TypeTag);
    static LiquidityPoolParser(data: any, typeTag: TypeTag, repo: AptosParserRepo): LiquidityPool;
    static load(repo: AptosParserRepo, client: AptosClient, address: HexString, typeParams: TypeTag[]): Promise<LiquidityPool>;
    static loadByApp(app: $.AppType, address: HexString, typeParams: TypeTag[]): Promise<LiquidityPool>;
    static makeTag($p: TypeTag[]): StructTag;
    loadFullState(app: $.AppType): Promise<void>;
}
export declare function assert_no_emergency_($c: AptosDataCache): void;
export declare function assert_pool_unlocked_($c: AptosDataCache, $p: TypeTag[]): void;
export declare function get_decimals_scales_($c: AptosDataCache, $p: TypeTag[]): [U64, U64];
export declare function get_fee_($c: AptosDataCache, $p: TypeTag[]): U64;
export declare function get_fees_config_($c: AptosDataCache, $p: TypeTag[]): [U64, U64];
export declare function get_reserves_size_($c: AptosDataCache, $p: TypeTag[]): [U64, U64];
export declare function is_emergency_($c: AptosDataCache): boolean;
export declare function loadParsers(repo: AptosParserRepo): void;
export declare class App {
    client: AptosClient;
    repo: AptosParserRepo;
    cache: AptosLocalCache;
    constructor(client: AptosClient, repo: AptosParserRepo, cache: AptosLocalCache);
    get moduleAddress(): HexString;
    get moduleName(): string;
    get IsEmergency(): typeof IsEmergency;
    loadIsEmergency(owner: HexString, loadFull?: boolean, fillCache?: boolean): Promise<IsEmergency>;
    get LiquidityPool(): typeof LiquidityPool;
    loadLiquidityPool(owner: HexString, $p: TypeTag[], loadFull?: boolean, fillCache?: boolean): Promise<LiquidityPool>;
}
//# sourceMappingURL=liquidity_pool.d.ts.map